@@ -31,10 +31,12 @@ modules = [
     "batoto",
     "bbc",
     "behance",
+    "bellazon",
     "bilibili",
     "blogger",
     "bluesky",
     "boosty",
+    "booth",
     "bunkr",
     "catbox",
     "chevereto",
@@ -43,6 +45,7 @@ modules = [
     "comick",
     "comicvine",
     "cyberdrop",
+    "cyberfile",
     "danbooru",
     "dankefuerslesen",
     "desktopography",
@@ -55,6 +58,7 @@ modules = [
     "exhentai",
     "facebook",
     "fanbox",
+    "fansly",
     "fantia",
     "fapello",
     "fapachi",
@@ -69,6 +73,7 @@ modules = [
     "girlswithmuscle",
     "gofile",
     "hatenablog",
+    "hdoujin",
     "hentai2read",
     "hentaicosplays",
     "hentaifoundry",
@@ -84,6 +89,7 @@ modules = [
     "imagefap",
     "imgbb",
     "imgbox",
+    "imgpile",
     "imgth",
     "imgur",
     "imhentai",
@@ -114,6 +120,7 @@ modules = [
     "manganelo",
     "mangapark",
     "mangaread",
+    "mangataro",
     "mangoxo",
     "misskey",
     "motherless",
@@ -168,7 +175,9 @@ modules = [
     "senmanga",
     "sexcom",
     "shimmie2",
+    "simpcity",
     "simplyhentai",
+    "sizebooru",
     "skeb",
     "slickpic",
     "slideshare",
@@ -182,12 +191,14 @@ modules = [
     "tcbscans",
     "telegraph",
     "tenor",
+    "thehentaiworld",
     "tiktok",
     "tmohentai",
     "toyhouse",
     "tsumino",
     "tumblr",
     "tumblrgallery",
+    "tungsten",
     "twibooru",
     "twitter",
     "urlgalleries",
@@ -210,12 +221,12 @@ modules = [
     "wikiart",
     "wikifeet",
     "wikimedia",
+    "xasiat",
     "xfolio",
     "xhamster",
     "xvideos",
     "yiffverse",
     "zerochan",
-    "zzup",
     "booru",
     "moebooru",
     "foolfuuka",
