@@ -10,12 +10,15 @@ from . import lolisafe
 from .common import Message
 from .. import text
 
+BASE_PATTERN = r"(?:https?://)?(?:www\.)?cyberdrop\.(?:me|to)"
+
 
 class CyberdropAlbumExtractor(lolisafe.LolisafeAlbumExtractor):
+    """Extractor for cyberdrop albums"""
     category = "cyberdrop"
     root = "https://cyberdrop.me"
     root_api = "https://api.cyberdrop.me"
-    pattern = r"(?:https?://)?(?:www\.)?cyberdrop\.(?:me|to)/a/([^/?#]+)"
+    pattern = BASE_PATTERN + r"/a/([^/?#]+)"
     example = "https://cyberdrop.me/a/ID"
 
     def items(self):
@@ -29,7 +32,7 @@ class CyberdropAlbumExtractor(lolisafe.LolisafeAlbumExtractor):
             yield Message.Url, file["url"], file
 
     def fetch_album(self, album_id):
-        url = "{}/a/{}".format(self.root, album_id)
+        url = f"{self.root}/a/{album_id}"
         page = self.request(url).text
         extr = text.extract_from(page)
 
@@ -40,7 +43,7 @@ class CyberdropAlbumExtractor(lolisafe.LolisafeAlbumExtractor):
         extr('id="title"', "")
 
         album = {
-            "album_id"   : self.album_id,
+            "album_id"   : album_id,
             "album_name" : text.unescape(extr('title="', '"')),
             "album_size" : text.parse_bytes(extr(
                 '<p class="title">', "B")),
@@ -57,9 +60,9 @@ class CyberdropAlbumExtractor(lolisafe.LolisafeAlbumExtractor):
     def _extract_files(self, file_ids):
         for file_id in file_ids:
             try:
-                url = "{}/api/file/info/{}".format(self.root_api, file_id)
-                file = self.request(url).json()
-                auth = self.request(file["auth_url"]).json()
+                url = f"{self.root_api}/api/file/info/{file_id}"
+                file = self.request_json(url)
+                auth = self.request_json(file["auth_url"])
                 file["url"] = auth["url"]
             except Exception as exc:
                 self.log.warning("%s (%s: %s)",
@@ -67,3 +70,20 @@ class CyberdropAlbumExtractor(lolisafe.LolisafeAlbumExtractor):
                 continue
 
             yield file
+
+
+class CyberdropMediaExtractor(CyberdropAlbumExtractor):
+    """Extractor for cyberdrop media links"""
+    subcategory = "media"
+    directory_fmt = ("{category}",)
+    pattern = BASE_PATTERN + r"/f/([^/?#]+)"
+    example = "https://cyberdrop.me/f/ID"
+
+    def fetch_album(self, album_id):
+        return self._extract_files((album_id,)), {
+            "album_id"   : "",
+            "album_name" : "",
+            "album_size" : -1,
+            "description": "",
+            "count"      : 1,
+        }
