@@ -430,13 +430,15 @@ class DeviantartExtractor(Extractor):
             if children := content.get("content"):
                 html.append('<p style="')
 
-                attrs = content["attrs"]
-                if attrs.get("textAlign"):
-                    html.append("text-align:")
-                    html.append(attrs["textAlign"])
-                    html.append(";")
-                self._tiptap_process_indentation(html, attrs)
-                html.append('">')
+                if attrs := content.get("attrs"):
+                    if align := attrs.get("textAlign"):
+                        html.append("text-align:")
+                        html.append(align)
+                        html.append(";")
+                    self._tiptap_process_indentation(html, attrs)
+                    html.append('">')
+                else:
+                    html.append('margin-inline-start:0px">')
 
                 for block in children:
                     self._tiptap_process_content(html, block)
@@ -1354,7 +1356,8 @@ class DeviantartSearchExtractor(DeviantartExtractor):
 
     def _search_html(self, params):
         url = self.root + "/search"
-
+        find = text.re(r'''href="https://www.deviantart.com/([^/?#]+)'''
+                       r'''/(art|journal)/(?:[^"]+-)?(\d+)''').findall
         while True:
             response = self.request(url, params=params)
 
@@ -1362,12 +1365,11 @@ class DeviantartSearchExtractor(DeviantartExtractor):
                 raise exception.AbortExtraction("HTTP redirect to login page")
             page = response.text
 
-            for dev in DeviantartDeviationExtractor.pattern.findall(
-                    page)[2::3]:
+            for user, type, did in find(page)[:-3:3]:
                 yield {
-                    "deviationId": dev[3],
-                    "author": {"username": dev[0]},
-                    "isJournal": dev[2] == "journal",
+                    "deviationId": did,
+                    "author": {"username": user},
+                    "isJournal": type == "journal",
                 }
 
             cursor = text.extr(page, r'\"cursor\":\"', '\\',)
