@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2020-2023 Mike Fährmann
+# Copyright 2020-2025 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -103,17 +103,17 @@ class InkbunnyPoolExtractor(InkbunnyExtractor):
     subcategory = "pool"
     pattern = (BASE_PATTERN + r"/(?:"
                r"poolview_process\.php\?pool_id=(\d+)|"
-               r"submissionsviewall\.php\?([^#]+&mode=pool&[^#]+))")
+               r"submissionsviewall\.php"
+               r"\?((?:[^#]+&)?mode=pool(?:&[^#]+)?))")
     example = "https://inkbunny.net/poolview_process.php?pool_id=12345"
 
     def __init__(self, match):
         InkbunnyExtractor.__init__(self, match)
-        pid = match.group(1)
-        if pid:
+        if pid := match[1]:
             self.pool_id = pid
             self.orderby = "pool_order"
         else:
-            params = text.parse_query(match.group(2))
+            params = text.parse_query(match[2])
             self.pool_id = params.get("pool_id")
             self.orderby = params.get("orderby", "pool_order")
 
@@ -131,25 +131,36 @@ class InkbunnyPoolExtractor(InkbunnyExtractor):
 class InkbunnyFavoriteExtractor(InkbunnyExtractor):
     """Extractor for inkbunny user favorites"""
     subcategory = "favorite"
+    directory_fmt = ("{category}", "{favs_username!l}", "Favorites")
     pattern = (BASE_PATTERN + r"/(?:"
                r"userfavorites_process\.php\?favs_user_id=(\d+)|"
-               r"submissionsviewall\.php\?([^#]+&mode=userfavs&[^#]+))")
+               r"submissionsviewall\.php"
+               r"\?((?:[^#]+&)?mode=userfavs(?:&[^#]+)?))")
     example = ("https://inkbunny.net/userfavorites_process.php"
                "?favs_user_id=12345")
 
     def __init__(self, match):
         InkbunnyExtractor.__init__(self, match)
-        uid = match.group(1)
-        if uid:
+        if uid := match[1]:
             self.user_id = uid
             self.orderby = self.config("orderby", "fav_datetime")
         else:
-            params = text.parse_query(match.group(2))
+            params = text.parse_query(match[2])
             self.user_id = params.get("user_id")
             self.orderby = params.get("orderby", "fav_datetime")
 
     def metadata(self):
-        return {"favs_user_id": self.user_id}
+        # Lookup fav user ID as username
+        url = (f"{self.root}/userfavorites_process.php"
+               f"?favs_user_id={self.user_id}")
+        page = self.request(url).text
+        user_link = text.extr(page, '<a rel="author"', '</a>')
+        favs_username = text.extr(user_link, 'href="/', '"')
+
+        return {
+            "favs_user_id": self.user_id,
+            "favs_username": favs_username,
+        }
 
     def posts(self):
         params = {
@@ -161,17 +172,37 @@ class InkbunnyFavoriteExtractor(InkbunnyExtractor):
         return self.api.search(params)
 
 
+class InkbunnyUnreadExtractor(InkbunnyExtractor):
+    """Extractor for unread inkbunny submissions"""
+    subcategory = "unread"
+    pattern = (BASE_PATTERN + r"/submissionsviewall\.php"
+               r"\?((?:[^#]+&)?mode=unreadsubs(?:&[^#]+)?)")
+    example = ("https://inkbunny.net/submissionsviewall.php"
+               "?text=&mode=unreadsubs&type=")
+
+    def __init__(self, match):
+        InkbunnyExtractor.__init__(self, match)
+        self.params = text.parse_query(match[1])
+
+    def posts(self):
+        params = self.params.copy()
+        params.pop("rid", None)
+        params.pop("mode", None)
+        params["unread_submissions"] = "yes"
+        return self.api.search(params)
+
+
 class InkbunnySearchExtractor(InkbunnyExtractor):
     """Extractor for inkbunny search results"""
     subcategory = "search"
-    pattern = (BASE_PATTERN +
-               r"/submissionsviewall\.php\?([^#]+&mode=search&[^#]+)")
+    pattern = (BASE_PATTERN + r"/submissionsviewall\.php"
+               r"\?((?:[^#]+&)?mode=search(?:&[^#]+)?)")
     example = ("https://inkbunny.net/submissionsviewall.php"
                "?text=TAG&mode=search&type=")
 
     def __init__(self, match):
         InkbunnyExtractor.__init__(self, match)
-        self.params = text.parse_query(match.group(1))
+        self.params = text.parse_query(match[1])
 
     def metadata(self):
         return {"search": self.params}
@@ -185,10 +216,9 @@ class InkbunnySearchExtractor(InkbunnyExtractor):
         params["dayslimit"] = pop("days", None)
         params["username"] = pop("artist", None)
 
-        favsby = pop("favsby", None)
-        if favsby:
+        if favsby := pop("favsby", None):
             # get user_id from user profile
-            url = "{}/{}".format(self.root, favsby)
+            url = f"{self.root}/{favsby}"
             page = self.request(url).text
             user_id = text.extr(page, "?user_id=", "'")
             params["favs_user_id"] = user_id.partition("&")[0]
@@ -201,14 +231,15 @@ class InkbunnyFollowingExtractor(InkbunnyExtractor):
     subcategory = "following"
     pattern = (BASE_PATTERN + r"/(?:"
                r"watchlist_process\.php\?mode=watching&user_id=(\d+)|"
-               r"usersviewall\.php\?([^#]+&mode=watching&[^#]+))")
+               r"usersviewall\.php"
+               r"\?((?:[^#]+&)?mode=watching(?:&[^#]+)?))")
     example = ("https://inkbunny.net/watchlist_process.php"
                "?mode=watching&user_id=12345")
 
     def __init__(self, match):
         InkbunnyExtractor.__init__(self, match)
-        self.user_id = match.group(1) or \
-            text.parse_query(match.group(2)).get("user_id")
+        self.user_id = match[1] or \
+            text.parse_query(match[2]).get("user_id")
 
     def items(self):
         url = self.root + "/watchlist_process.php"
@@ -223,14 +254,12 @@ class InkbunnyFollowingExtractor(InkbunnyExtractor):
         data = {"_extractor": InkbunnyUserExtractor}
 
         while True:
-            cnt = 0
             for user in text.extract_iter(
                     page, '<a class="widget_userNameSmall" href="', '"',
                     page.index('id="changethumboriginal_form"')):
-                cnt += 1
                 yield Message.Queue, self.root + user, data
 
-            if cnt < 20:
+            if "<a title='next page' " not in page:
                 return
             params["page"] += 1
             page = self.request(url, params=params).text
@@ -244,7 +273,7 @@ class InkbunnyPostExtractor(InkbunnyExtractor):
 
     def __init__(self, match):
         InkbunnyExtractor.__init__(self, match)
-        self.submission_id = match.group(1)
+        self.submission_id = match[1]
 
     def posts(self):
         submissions = self.api.detail(({"submission_id": self.submission_id},))
@@ -272,6 +301,7 @@ class InkbunnyAPI():
         params = {
             "submission_ids": ",".join(ids),
             "show_description": "yes",
+            "show_pools": "yes",
         }
 
         submissions = [None] * len(ids)
@@ -306,16 +336,19 @@ class InkbunnyAPI():
 
     def _call(self, endpoint, params):
         url = "https://inkbunny.net/api_" + endpoint + ".php"
-        params["sid"] = self.session_id
-        data = self.extractor.request(url, params=params).json()
 
-        if "error_code" in data:
+        while True:
+            params["sid"] = self.session_id
+            data = self.extractor.request_json(url, params=params)
+
+            if "error_code" not in data:
+                return data
+
             if str(data["error_code"]) == "2":
                 self.authenticate(invalidate=True)
-                return self._call(endpoint, params)
-            raise exception.StopExtraction(data.get("error_message"))
+                continue
 
-        return data
+            raise exception.AbortExtraction(data.get("error_message"))
 
     def _pagination_search(self, params):
         params["page"] = 1
@@ -324,6 +357,9 @@ class InkbunnyAPI():
 
         while True:
             data = self._call("search", params)
+            if not data["submissions"]:
+                return
+
             yield from self.detail(data["submissions"])
 
             if data["page"] >= data["pages_count"]:
@@ -334,13 +370,13 @@ class InkbunnyAPI():
             params["page"] += 1
 
 
-@cache(maxage=360*24*3600, keyarg=1)
+@cache(maxage=365*86400, keyarg=1)
 def _authenticate_impl(api, username, password):
     api.extractor.log.info("Logging in as %s", username)
 
     url = "https://inkbunny.net/api_login.php"
     data = {"username": username, "password": password}
-    data = api.extractor.request(url, method="POST", data=data).json()
+    data = api.extractor.request_json(url, method="POST", data=data)
 
     if "sid" not in data:
         raise exception.AuthenticationError(data.get("error_message"))
