@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2015-2023 Mike Fährmann
+# Copyright 2015-2025 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -9,7 +9,7 @@
 """Recursive extractor"""
 
 from .common import Extractor, Message
-import re
+from .. import text, util
 
 
 class RecursiveExtractor(Extractor):
@@ -25,7 +25,7 @@ class RecursiveExtractor(Extractor):
             with open(url[7:]) as fp:
                 page = fp.read()
         else:
-            page = self.request(url).text
+            page = self.request(text.ensure_http_scheme(url)).text
 
-        for match in re.finditer(r"https?://[^\s\"']+", page):
-            yield Message.Queue, match.group(0), {}
+        for match in util.re(r"https?://[^\s\"']+").finditer(page):
+            yield Message.Queue, match[0], {}
