@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2018-2022 Mike Fährmann
+# Copyright 2018-2025 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -8,19 +8,19 @@
 
 """Extractors for https://wallhaven.cc/"""
 
-from .common import Extractor, Message
-from .. import text
+from .common import Extractor, Message, Dispatch
+from .. import text, exception
 
 
 class WallhavenExtractor(Extractor):
     """Base class for wallhaven extractors"""
     category = "wallhaven"
+    root = "https://wallhaven.cc"
     filename_fmt = "{category}_{id}_{resolution}.{extension}"
     archive_fmt = "{id}"
-    root = "https://wallhaven.cc"
+    request_interval = 1.4
 
-    def __init__(self, match):
-        Extractor.__init__(self, match)
+    def _init(self):
         self.api = WallhavenAPI(self)
 
     def items(self):
@@ -39,8 +39,7 @@ class WallhavenExtractor(Extractor):
         """Return general metadata"""
         return ()
 
-    @staticmethod
-    def _transform(wp):
+    def _transform(self, wp):
         wp["url"] = wp.pop("path")
         if "tags" in wp:
             wp["tags"] = [t["name"] for t in wp["tags"]]
@@ -54,25 +53,17 @@ class WallhavenExtractor(Extractor):
 class WallhavenSearchExtractor(WallhavenExtractor):
     """Extractor for search results on wallhaven.cc"""
     subcategory = "search"
-    directory_fmt = ("{category}", "{search[q]}")
+    directory_fmt = ("{category}", "{search[tags]}")
     archive_fmt = "s_{search[q]}_{id}"
     pattern = r"(?:https?://)?wallhaven\.cc/search(?:/?\?([^#]+))?"
-    test = (
-        ("https://wallhaven.cc/search?q=touhou"),
-        (("https://wallhaven.cc/search?q=id%3A87"
-          "&categories=111&purity=100&sorting=date_added&order=asc&page=3"), {
-            "pattern": (r"https://w\.wallhaven\.cc"
-                        r"/full/\w\w/wallhaven-\w+\.\w+"),
-            "count": "<= 30",
-        }),
-    )
+    example = "https://wallhaven.cc/search?q=QUERY"
 
     def __init__(self, match):
         WallhavenExtractor.__init__(self, match)
-        self.params = text.parse_query(match.group(1))
+        self.params = text.parse_query(match[1])
 
     def wallpapers(self):
-        return self.api.search(self.params.copy())
+        return self.api.search(self.params)
 
     def metadata(self):
         return {"search": self.params}
@@ -83,9 +74,7 @@ class WallhavenCollectionExtractor(WallhavenExtractor):
     subcategory = "collection"
     directory_fmt = ("{category}", "{username}", "{collection_id}")
     pattern = r"(?:https?://)?wallhaven\.cc/user/([^/?#]+)/favorites/(\d+)"
-    test = ("https://wallhaven.cc/user/AksumkA/favorites/74", {
-        "count": ">= 50",
-    })
+    example = "https://wallhaven.cc/user/USER/favorites/12345"
 
     def __init__(self, match):
         WallhavenExtractor.__init__(self, match)
@@ -98,18 +87,13 @@ class WallhavenCollectionExtractor(WallhavenExtractor):
         return {"username": self.username, "collection_id": self.collection_id}
 
 
-class WallhavenUserExtractor(WallhavenExtractor):
+class WallhavenUserExtractor(Dispatch, WallhavenExtractor):
     """Extractor for a wallhaven user"""
-    subcategory = "user"
     pattern = r"(?:https?://)?wallhaven\.cc/user/([^/?#]+)/?$"
-    test = ("https://wallhaven.cc/user/AksumkA/",)
-
-    def __init__(self, match):
-        WallhavenExtractor.__init__(self, match)
-        self.username = match.group(1)
+    example = "https://wallhaven.cc/user/USER"
 
     def items(self):
-        base = "{}/user/{}/".format(self.root, self.username)
+        base = f"{self.root}/user/{self.groups[0]}/"
         return self._dispatch_extractors((
             (WallhavenUploadsExtractor    , base + "uploads"),
             (WallhavenCollectionsExtractor, base + "favorites"),
@@ -120,20 +104,17 @@ class WallhavenCollectionsExtractor(WallhavenExtractor):
     """Extractor for all collections of a wallhaven user"""
     subcategory = "collections"
     pattern = r"(?:https?://)?wallhaven\.cc/user/([^/?#]+)/favorites/?$"
-    test = ("https://wallhaven.cc/user/AksumkA/favorites", {
-        "pattern": WallhavenCollectionExtractor.pattern,
-        "count": 4,
-    })
+    example = "https://wallhaven.cc/user/USER/favorites"
 
     def __init__(self, match):
         WallhavenExtractor.__init__(self, match)
-        self.username = match.group(1)
+        self.username = match[1]
 
     def items(self):
+        base = f"{self.root}/user/{self.username}/favorites/"
         for collection in self.api.collections(self.username):
             collection["_extractor"] = WallhavenCollectionExtractor
-            url = "https://wallhaven.cc/user/{}/favorites/{}".format(
-                self.username, collection["id"])
+            url = f"{base}{collection['id']}"
             yield Message.Queue, url, collection
 
 
@@ -143,20 +124,15 @@ class WallhavenUploadsExtractor(WallhavenExtractor):
     directory_fmt = ("{category}", "{username}")
     archive_fmt = "u_{username}_{id}"
     pattern = r"(?:https?://)?wallhaven\.cc/user/([^/?#]+)/uploads"
-    test = ("https://wallhaven.cc/user/AksumkA/uploads", {
-        "pattern": (r"https://[^.]+\.wallhaven\.cc"
-                    r"/full/\w\w/wallhaven-\w+\.\w+"),
-        "range": "1-100",
-        "count": 100,
-    })
+    example = "https://wallhaven.cc/user/USER/uploads"
 
     def __init__(self, match):
         WallhavenExtractor.__init__(self, match)
-        self.username = match.group(1)
+        self.username = match[1]
 
     def wallpapers(self):
         params = {"q": "@" + self.username}
-        return self.api.search(params.copy())
+        return self.api.search(params)
 
     def metadata(self):
         return {"username": self.username}
@@ -167,45 +143,11 @@ class WallhavenImageExtractor(WallhavenExtractor):
     subcategory = "image"
     pattern = (r"(?:https?://)?(?:wallhaven\.cc/w/|whvn\.cc/"
                r"|w\.wallhaven\.cc/[a-z]+/\w\w/wallhaven-)(\w+)")
-    test = (
-        ("https://wallhaven.cc/w/01w334", {
-            "pattern": (r"https://[^.]+\.wallhaven\.cc"
-                        r"/full/01/wallhaven-01w334\.jpg"),
-            "content": "497212679383a465da1e35bd75873240435085a2",
-            "keyword": {
-                "id"         : "01w334",
-                "width"      : 1920,
-                "height"     : 1200,
-                "resolution" : "1920x1200",
-                "ratio"      : "1.6",
-                "colors"     : list,
-                "tags"       : list,
-                "file_size"  : 278799,
-                "file_type"  : "image/jpeg",
-                "purity"     : "sfw",
-                "short_url"  : "https://whvn.cc/01w334",
-                "source"     : str,
-                "uploader"   : {
-                    "group"    : "Owner/Developer",
-                    "username" : "AksumkA",
-                },
-                "date"       : "dt:2014-08-31 06:17:19",
-                "wh_category": "anime",
-                "views"      : int,
-                "favorites"  : int,
-            },
-        }),
-        # NSFW
-        ("https://wallhaven.cc/w/dge6v3", {
-            "url": "e4b802e70483f659d790ad5d0bd316245badf2ec",
-        }),
-        ("https://whvn.cc/01w334"),
-        ("https://w.wallhaven.cc/full/01/wallhaven-01w334.jpg"),
-    )
+    example = "https://wallhaven.cc/w/ID"
 
     def __init__(self, match):
         WallhavenExtractor.__init__(self, match)
-        self.wallpaper_id = match.group(1)
+        self.wallpaper_id = match[1]
 
     def wallpapers(self):
         return (self.api.info(self.wallpaper_id),)
@@ -233,7 +175,7 @@ class WallhavenAPI():
         return self._call(endpoint)["data"]
 
     def collection(self, username, collection_id):
-        endpoint = "/v1/collections/{}/{}".format(username, collection_id)
+        endpoint = f"/v1/collections/{username}/{collection_id}"
         return self._pagination(endpoint)
 
     def collections(self, username):
@@ -246,25 +188,53 @@ class WallhavenAPI():
 
     def _call(self, endpoint, params=None):
         url = "https://wallhaven.cc/api" + endpoint
-        return self.extractor.request(
-            url, headers=self.headers, params=params).json()
+
+        while True:
+            response = self.extractor.request(
+                url, params=params, headers=self.headers, fatal=None)
+
+            if response.status_code < 400:
+                return response.json()
+            if response.status_code == 429:
+                self.extractor.wait(seconds=60)
+                continue
+
+            self.extractor.log.debug("Server response: %s", response.text)
+            raise exception.AbortExtraction(
+                f"API request failed "
+                f"({response.status_code} {response.reason})")
 
     def _pagination(self, endpoint, params=None, metadata=None):
         if params is None:
+            params_ptr = None
             params = {}
+        else:
+            params_ptr = params
+            params = params.copy()
         if metadata is None:
             metadata = self.extractor.config("metadata")
 
         while True:
             data = self._call(endpoint, params)
 
+            meta = data.get("meta")
+            if params_ptr is not None:
+                if meta and "query" in meta:
+                    query = meta["query"]
+                    if isinstance(query, dict):
+                        params_ptr["tags"] = query.get("tag")
+                        params_ptr["tag_id"] = query.get("id")
+                    else:
+                        params_ptr["tags"] = query
+                        params_ptr["tag_id"] = 0
+                params_ptr = None
+
             if metadata:
                 for wp in data["data"]:
                     yield self.info(str(wp["id"]))
             else:
                 yield from data["data"]
 
-            meta = data.get("meta")
             if not meta or meta["current_page"] >= meta["last_page"]:
                 return
             params["page"] = meta["current_page"] + 1
