@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2017-2023 Mike Fährmann
+# Copyright 2017-2025 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -11,6 +11,9 @@
 from .common import GalleryExtractor, Extractor, Message
 from .. import text, util
 
+BASE_PATTERN = (r"(?:https?://)?(?:www\.)?xvideos\.com"
+                r"/(?:profiles|(?:amateur-|model-)?channels)")
+
 
 class XvideosBase():
     """Base class for xvideos extractors"""
@@ -25,26 +28,23 @@ class XvideosGalleryExtractor(XvideosBase, GalleryExtractor):
                      "{gallery[id]} {gallery[title]}")
     filename_fmt = "{category}_{gallery[id]}_{num:>03}.{extension}"
     archive_fmt = "{gallery[id]}_{num}"
-    pattern = (r"(?:https?://)?(?:www\.)?xvideos\.com"
-               r"/(?:profiles|amateur-channels|model-channels)"
-               r"/([^/?#]+)/photos/(\d+)")
+    pattern = BASE_PATTERN + r"/([^/?#]+)/photos/(\d+)"
     example = "https://www.xvideos.com/profiles/USER/photos/12345"
 
     def __init__(self, match):
         self.user, self.gallery_id = match.groups()
-        url = "{}/profiles/{}/photos/{}".format(
-            self.root, self.user, self.gallery_id)
+        url = f"{self.root}/profiles/{self.user}/photos/{self.gallery_id}"
         GalleryExtractor.__init__(self, match, url)
 
     def metadata(self, page):
         extr = text.extract_from(page)
-        title = extr('"title":"', '"')
         user = {
             "id"     : text.parse_int(extr('"id_user":', ',')),
             "display": extr('"display":"', '"'),
             "sex"    : extr('"sex":"', '"'),
             "name"   : self.user,
         }
+        title = extr('"title":"', '"')
         user["description"] = extr(
             '<small class="mobile-hide">', '</small>').strip()
         tags = extr('<em>Tagged:</em>', '<').strip()
@@ -58,30 +58,43 @@ class XvideosGalleryExtractor(XvideosBase, GalleryExtractor):
             },
         }
 
-    @staticmethod
-    def images(page):
-        """Return a list of all image urls for this gallery"""
-        return [
+    def images(self, page):
+        results = [
             (url, None)
             for url in text.extract_iter(
                 page, '<a class="embed-responsive-item" href="', '"')
         ]
 
+        if not results:
+            return
+
+        while len(results) % 500 == 0:
+            path = text.rextr(page, ' href="', '"', page.find(">Next</"))
+            if not path:
+                break
+            page = self.request(self.root + path).text
+            results.extend(
+                (url, None)
+                for url in text.extract_iter(
+                    page, '<a class="embed-responsive-item" href="', '"')
+            )
+
+        return results
+
 
 class XvideosUserExtractor(XvideosBase, Extractor):
     """Extractor for user profiles on xvideos.com"""
     subcategory = "user"
     categorytransfer = True
-    pattern = (r"(?:https?://)?(?:www\.)?xvideos\.com"
-               r"/profiles/([^/?#]+)/?(?:#.*)?$")
+    pattern = BASE_PATTERN + r"/([^/?#]+)/?(?:#.*)?$"
     example = "https://www.xvideos.com/profiles/USER"
 
     def __init__(self, match):
         Extractor.__init__(self, match)
-        self.user = match.group(1)
+        self.user = match[1]
 
     def items(self):
-        url = "{}/profiles/{}".format(self.root, self.user)
+        url = f"{self.root}/profiles/{self.user}"
         page = self.request(url, notfound=self.subcategory).text
         data = util.json_loads(text.extr(
             page, "xv.conf=", ";</script>"))["data"]
@@ -102,7 +115,7 @@ class XvideosUserExtractor(XvideosBase, Extractor):
         ]
         galleries.sort(key=lambda x: x["id"])
 
+        base = f"{self.root}/profiles/{self.user}/photos/"
         for gallery in galleries:
-            url = "https://www.xvideos.com/profiles/{}/photos/{}".format(
-                self.user, gallery["id"])
+            url = f"{base}{gallery['id']}"
             yield Message.Queue, url, gallery
